/*************************************************************************/
/* Copyright (C) 2014 - Daniel Rubio Bonilla                             */
/*                      danielrubiob [at] gmail [dot] com                */
/*                                                                       */
/* This program is free software: you can redistribute it and/or modify  */
/* it under the terms of the GNU General Public License as published by  */
/* the Free Software Foundation, either version 3 of the License, or     */
/* (at your option) any later version.                                   */
/*                                                                       */
/* This program is distributed in the hope that it will be useful,       */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of        */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         */
/* GNU General Public License for more details.                          */
/*                                                                       */
/* You should have received a copy of the GNU General Public License     */
/* along with this program.  If not, see <http://www.gnu.org/licenses/>. */
/*************************************************************************/

#include <fcntl.h>
#include <stdio.h>
#include <linux/i2c-dev.h>

#include "i2c-driver.h"

int i2c_dev_open(struct i2c_dev *dev, int adapter) {
  char file[16];

  snprintf(file, 15, "/dev/i2c-%d", adapter); 
  file[15] = '\0';
  dev->fd = open(file, O_RDWR);

  return dev->fd;
}

int i2c_dev_close(struct i2c_dev *dev) {
  return close(dev->fd);
}

int i2c_dev_write(struct i2c_dev *dev, uint8_t *buf, int n) {
  struct i2c_rdwr_ioctl_data msgs;
  struct i2c_msg msg;

  msgs.nmsgs = 1;
  msgs.msgs = &msg;

  msg.addr  = dev->addr;
  if(dev->tba)
    msg.flags = I2C_M_TEN;
  else
    msg.flags = 0;
  msg.len = n;
  msg.buf = (char *)buf;

  return ioctl(dev->fd, I2C_RDWR, &msgs);
}

int i2c_dev_read(struct i2c_dev *dev, uint8_t *buf, int n) {
  struct i2c_rdwr_ioctl_data msgs;
  struct i2c_msg msg;

  msgs.nmsgs = 1;
  msgs.msgs = &msg;

  msg.addr  = dev->addr;
  if(dev->tba)
    msg.flags = I2C_M_TEN | I2C_M_RD;
  else
    msg.flags = I2C_M_RD;
  msg.len = n;
  msg.buf = (char *)buf;

  return ioctl(dev->fd, I2C_RDWR, &msgs);
}

int i2c_dev_wrrs(struct i2c_dev *dev, uint8_t *wbuf, int wn,
		 uint8_t *rbuf, int rn) {
  struct i2c_rdwr_ioctl_data msgs;
  struct i2c_msg msg[2];

  msgs.nmsgs = 2;
  msgs.msgs = msg;

  msg[0].addr  = dev->addr;
  if(dev->tba)
    msg[0].flags = I2C_M_TEN;
  else
    msg[0].flags = 0;
  msg[0].len = wn;
  msg[0].buf = (char *)wbuf;

  msg[1].addr  = dev->addr;
  if(dev->tba)
    msg[1].flags = I2C_M_TEN | I2C_M_RD;
  else
    msg[1].flags = I2C_M_RD;
  msg[1].len = rn;
  msg[1].buf = (char *)rbuf;

  return ioctl(dev->fd, I2C_RDWR, &msgs);
}

